@@ -51,6 +51,8 @@
 # include <strings.h>
 #endif
 
+#include "xalloc.h"
+
 #include "gettext.h"
 #include "api.h"
 #include "ext.h"
@@ -140,6 +142,27 @@ typedef struct _gss_mech_api_strict {
 	const gss_OID mech_type,
 	OM_uint32 * message_context,
 	gss_buffer_t status_string);
+  OM_uint32 (*acquire_cred)
+       (OM_uint32 * minor_status,
+	const gss_name_t desired_name,
+	OM_uint32 time_req,
+	const gss_OID_set desired_mechs,
+	gss_cred_usage_t cred_usage,
+	gss_cred_id_t * output_cred_handle,
+	gss_OID_set * actual_mechs,
+	OM_uint32 * time_rec);
+  OM_uint32 (*accept_sec_context)
+       (OM_uint32 * minor_status,
+	gss_ctx_id_t * context_handle,
+	const gss_cred_id_t acceptor_cred_handle,
+	const gss_buffer_t input_token_buffer,
+	const gss_channel_bindings_t input_chan_bindings,
+	gss_name_t * src_name,
+	gss_OID * mech_type,
+	gss_buffer_t output_token,
+	OM_uint32 * ret_flags,
+	OM_uint32 * time_rec,
+	gss_cred_id_t * delegated_cred_handle);
 } _gss_mech_api_desc, *_gss_mech_api_t;
 
 _gss_mech_api_t _gss_find_mech (gss_OID oid);
