@@ -59,7 +59,9 @@ _gss_mech_api_desc _gss_mech_apis[] = {
     gss_krb5_unwrap,
     _gss_dummy,
     _gss_dummy,
-    gss_krb5_display_status
+    gss_krb5_display_status,
+    gss_krb5_acquire_cred,
+    gss_krb5_accept_sec_context
   },
 #endif
   {
@@ -71,7 +73,9 @@ _gss_mech_api_desc _gss_mech_apis[] = {
     _gss_dummy,
     _gss_dummy,
     _gss_dummy,
-    _gss_dummy_display_status
+    _gss_dummy_display_status,
+    _gss_dummy,
+    _gss_dummy
   }
 };
 
